from MapTile import MapTile
from VMFNode import vectorToString
import os
import random

random.seed(43)
NUMBER_OF_TILES = 25

def chooseConection(connections):
  if len(connections) > 0:
    direction = random.choice(connections)
    portal = random.choice(direction[1])
    otherPortal = random.choice(direction[2])
    connection = (direction[0],portal,otherPortal)
    print "Chose connection",connection
    return connection
  else:
    return None

def addTile(base, tile, usedVectors):
  directions = base.findConnections(tile)
  for direction in directions:
    for portal in direction[1]:
      for otherPortal in direction[2]:
        connection = (direction[0],portal,otherPortal)
        vectors = base.findPortalsAndVector(tile, connection)
        vectorString = vectorToString(vectors[0])
        if not (vectorString == None or vectorString in usedVectors):
          usedVectors.append(vectorString)
          base.append(tile, connection, vectors)
          return True
  return False
    
def tryAddTile(base, tile, usedVectors):
  connections = base.findConnections(tile)
  connection = chooseConection(connections)
  if connection == None:
    return False
  vectors = base.findPortalsAndVector(tile, connection)
  vectorString = vectorToString(vectors[0])
  if vectorString == None or vectorString in usedVectors:
    return False
  else:
    usedVectors.append(vectorString)
    base.append(tile, connection, vectors)
    #print "Bounds:",base.map.root.GetBoundsRecurse()
    return True
    
def selectAndTryToAddTile(base, tiles, usedVectors):
  tile = random.choice(tiles)
  print "Chose tile",os.path.basename(tile.filename)
  return tryAddTile(base, tile, usedVectors)
    
def loadTiles(path):
  starts = []
  tiles = []
  finales = []
  listing = os.listdir(path)
  for filename in listing:
    if filename[-3:] == "vmf":
      print "Loading",os.path.basename(filename)
      maptile = MapTile()
      maptile.fromfile(path+filename)
      if filename[:5] == "start":
        starts.append(maptile)
      elif filename[:6] == "finale":
        finales.append(maptile)
      else:
        tiles.append(maptile)
  return (starts, tiles, finales)
    
starts, tiles, finales = loadTiles("tiles/office/")
base = random.choice(starts)
finale = random.choice(finales)

usedVectors = ["0 0 0"]
tilesAdded = 0
for i in range(NUMBER_OF_TILES*len(tiles)):
  if selectAndTryToAddTile(base, tiles, usedVectors):
    tilesAdded += 1
  if tilesAdded == NUMBER_OF_TILES:
    break
print tilesAdded,"tiles added"
    
if not addTile(base, finale, usedVectors):
  print "ERROR: Failed to append final \"finale\" tile."
else:
  pass
  
base.close()

file = open("combined.vmf","w")
file.write(base.map.root.ToStringRecurse(0))
file.close()

file = open("../../../left4dead2/cfg/combined.cfg","w")
file.write(base.generateNavMeshScript())
file.close()